@@ -1,4 +1,4 @@
-#!/usr/bin/python2.5
+#!/usr/bin/env python
 
 """
 DialCentral - Front end for Google's GoogleVoice service.
@@ -34,8 +34,8 @@ import gtk
 
 import gtk_toolbox
 import hildonize
-import gv_backend
-import null_backend
+from backends import gv_backend
+from backends import null_backend
 
 
 _moduleLogger = logging.getLogger("gv_views")
@@ -165,155 +165,6 @@ def collapse_message(message, maxCharsPerLine, maxLines):
 	return "\n".join(_collapse_message(messageLines, maxCharsPerLine, maxLines))
 
 
-class MergedAddressBook(object):
-	"""
-	Merger of all addressbooks
-	"""
-
-	def __init__(self, addressbookFactories, sorter = None):
-		self.__addressbookFactories = addressbookFactories
-		self.__addressbooks = None
-		self.__sort_contacts = sorter if sorter is not None else self.null_sorter
-
-	def clear_caches(self):
-		self.__addressbooks = None
-		for factory in self.__addressbookFactories:
-			factory.clear_caches()
-
-	def get_addressbooks(self):
-		"""
-		@returns Iterable of (Address Book Factory, Book Id, Book Name)
-		"""
-		yield self, "", ""
-
-	def open_addressbook(self, bookId):
-		return self
-
-	def contact_source_short_name(self, contactId):
-		if self.__addressbooks is None:
-			return ""
-		bookIndex, originalId = contactId.split("-", 1)
-		return self.__addressbooks[int(bookIndex)].contact_source_short_name(originalId)
-
-	@staticmethod
-	def factory_name():
-		return "All Contacts"
-
-	def get_contacts(self):
-		"""
-		@returns Iterable of (contact id, contact name)
-		"""
-		if self.__addressbooks is None:
-			self.__addressbooks = list(
-				factory.open_addressbook(id)
-				for factory in self.__addressbookFactories
-				for (f, id, name) in factory.get_addressbooks()
-			)
-		contacts = (
-			("-".join([str(bookIndex), contactId]), contactName)
-				for (bookIndex, addressbook) in enumerate(self.__addressbooks)
-					for (contactId, contactName) in addressbook.get_contacts()
-		)
-		sortedContacts = self.__sort_contacts(contacts)
-		return sortedContacts
-
-	def get_contact_details(self, contactId):
-		"""
-		@returns Iterable of (Phone Type, Phone Number)
-		"""
-		if self.__addressbooks is None:
-			return []
-		bookIndex, originalId = contactId.split("-", 1)
-		return self.__addressbooks[int(bookIndex)].get_contact_details(originalId)
-
-	@staticmethod
-	def null_sorter(contacts):
-		"""
-		Good for speed/low memory
-		"""
-		return contacts
-
-	@staticmethod
-	def basic_firtname_sorter(contacts):
-		"""
-		Expects names in "First Last" format
-		"""
-		contactsWithKey = [
-			(contactName.rsplit(" ", 1)[0], (contactId, contactName))
-				for (contactId, contactName) in contacts
-		]
-		contactsWithKey.sort()
-		return (contactData for (lastName, contactData) in contactsWithKey)
-
-	@staticmethod
-	def basic_lastname_sorter(contacts):
-		"""
-		Expects names in "First Last" format
-		"""
-		contactsWithKey = [
-			(contactName.rsplit(" ", 1)[-1], (contactId, contactName))
-				for (contactId, contactName) in contacts
-		]
-		contactsWithKey.sort()
-		return (contactData for (lastName, contactData) in contactsWithKey)
-
-	@staticmethod
-	def reversed_firtname_sorter(contacts):
-		"""
-		Expects names in "Last, First" format
-		"""
-		contactsWithKey = [
-			(contactName.split(", ", 1)[-1], (contactId, contactName))
-				for (contactId, contactName) in contacts
-		]
-		contactsWithKey.sort()
-		return (contactData for (lastName, contactData) in contactsWithKey)
-
-	@staticmethod
-	def reversed_lastname_sorter(contacts):
-		"""
-		Expects names in "Last, First" format
-		"""
-		contactsWithKey = [
-			(contactName.split(", ", 1)[0], (contactId, contactName))
-				for (contactId, contactName) in contacts
-		]
-		contactsWithKey.sort()
-		return (contactData for (lastName, contactData) in contactsWithKey)
-
-	@staticmethod
-	def guess_firstname(name):
-		if ", " in name:
-			return name.split(", ", 1)[-1]
-		else:
-			return name.rsplit(" ", 1)[0]
-
-	@staticmethod
-	def guess_lastname(name):
-		if ", " in name:
-			return name.split(", ", 1)[0]
-		else:
-			return name.rsplit(" ", 1)[-1]
-
-	@classmethod
-	def advanced_firstname_sorter(cls, contacts):
-		contactsWithKey = [
-			(cls.guess_firstname(contactName), (contactId, contactName))
-				for (contactId, contactName) in contacts
-		]
-		contactsWithKey.sort()
-		return (contactData for (lastName, contactData) in contactsWithKey)
-
-	@classmethod
-	def advanced_lastname_sorter(cls, contacts):
-		contactsWithKey = [
-			(cls.guess_lastname(contactName), (contactId, contactName))
-				for (contactId, contactName) in contacts
-		]
-		contactsWithKey.sort()
-		return (contactData for (lastName, contactData) in contactsWithKey)
-
-
 class SmsEntryDialog(object):
 	"""
 	@todo Add multi-SMS messages like GoogleVoice
@@ -508,8 +359,7 @@ class SmsEntryDialog(object):
 					messagePart[0]
 					for messagePart in self._messagemodel
 				)
-				# For some reason this kills clipboard stuff
-				#self._clipboard.set_text(message)
+				self._clipboard.set_text(str(message))
 		except Exception, e:
 			_moduleLogger.exception(str(e))
 
@@ -1154,6 +1004,7 @@ class MessagesView(object):
 	MESSAGE_IDX = 3
 	MESSAGES_IDX = 4
 	FROM_ID_IDX = 5
+	MESSAGE_DATA_IDX = 6
 
 	NO_MESSAGES = "None"
 	VOICEMAIL_MESSAGES = "Voicemail"
@@ -1178,7 +1029,10 @@ class MessagesView(object):
 			gobject.TYPE_STRING, # message
 			object, # messages
 			gobject.TYPE_STRING, # from id
+			object, # message data
 		)
+		self._messagemodelfiltered = self._messagemodel.filter_new()
+		self._messagemodelfiltered.set_visible_func(self._is_message_visible)
 		self._messageview = widgetTree.get_widget("messages_view")
 		self._messageviewselection = None
 		self._onMessageviewRowActivatedId = 0
@@ -1210,7 +1064,7 @@ class MessagesView(object):
 
 	def enable(self):
 		assert self._backend.is_authed(), "Attempting to enable backend while not logged in"
-		self._messageview.set_model(self._messagemodel)
+		self._messageview.set_model(self._messagemodelfiltered)
 		self._messageview.set_headers_visible(False)
 		self._messageview.set_fixed_height_mode(False)
 
@@ -1279,7 +1133,14 @@ class MessagesView(object):
 		config.set(sectionName, "status", self._messageStatus)
 		config.set(sectionName, "type", self._messageType)
 
-	_MIN_MESSAGES_SHOWN = 4
+	def _is_message_visible(self, model, iter):
+		try:
+			message = model.get_value(iter, self.MESSAGE_DATA_IDX)
+			if message is None:
+				return False # this seems weird but oh well
+			return self._filter_messages(message, self._messageType, self._messageStatus)
+		except Exception, e:
+			self._errorDisplay.push_exception()
 
 	@classmethod
 	def _filter_messages(cls, message, type, status):
@@ -1303,6 +1164,8 @@ class MessagesView(object):
 
 		return isType and isStatus
 
+	_MIN_MESSAGES_SHOWN = 4
+
 	def _idly_populate_messageview(self):
 		with gtk_toolbox.gtk_lock():
 			banner = hildonize.show_busy_banner_start(self._window, "Loading Messages")
@@ -1321,12 +1184,11 @@ class MessagesView(object):
 					messageItems = []
 
 			messageItems = (
-				gv_backend.decorate_message(message)
+				(gv_backend.decorate_message(message), message)
 				for message in gv_backend.sort_messages(messageItems)
-				if self._filter_messages(message, self._messageType, self._messageStatus)
 			)
 
-			for contactId, header, number, relativeDate, messages in messageItems:
+			for (contactId, header, number, relativeDate, messages), messageData in messageItems:
 				prettyNumber = number[2:] if number.startswith("+1") else number
 				prettyNumber = make_pretty(prettyNumber)
 
@@ -1344,7 +1206,7 @@ class MessagesView(object):
 
 				number = make_ugly(number)
 
-				row = number, relativeDate, header, "\n".join(collapsedMessages), expandedMessages, contactId
+				row = number, relativeDate, header, "\n".join(collapsedMessages), expandedMessages, contactId, messageData
 				with gtk_toolbox.gtk_lock():
 					self._messagemodel.append(row)
 		except Exception, e:
@@ -1352,6 +1214,7 @@ class MessagesView(object):
 		finally:
 			with gtk_toolbox.gtk_lock():
 				hildonize.show_busy_banner_end(banner)
+				self._messagemodelfiltered.refilter()
 
 		return False
 
@@ -1417,7 +1280,7 @@ class MessagesView(object):
 			if selectedIndex != newSelectedIndex:
 				self._messageType = self.MESSAGE_TYPES[newSelectedIndex]
 				self._messageTypeButton.set_label(self._messageType)
-				self.update(True)
+				self._messagemodelfiltered.refilter()
 		except Exception, e:
 			self._errorDisplay.push_exception()
 
@@ -1438,13 +1301,17 @@ class MessagesView(object):
 			if selectedIndex != newSelectedIndex:
 				self._messageStatus = self.MESSAGE_STATUSES[newSelectedIndex]
 				self._messageStatusButton.set_label(self._messageStatus)
-				self.update(True)
+				self._messagemodelfiltered.refilter()
 		except Exception, e:
 			self._errorDisplay.push_exception()
 
 
 class ContactsView(object):
 
+	CONTACT_TYPE_IDX = 0
+	CONTACT_NAME_IDX = 1
+	CONTACT_ID_IDX = 2
+
 	def __init__(self, widgetTree, backend, errorDisplay):
 		self._errorDisplay = errorDisplay
 		self._backend = backend
@@ -1457,7 +1324,11 @@ class ContactsView(object):
 		self._bookSelectionButton = widgetTree.get_widget("addressbookSelectButton")
 
 		self._isPopulated = False
-		self._contactsmodel = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING)
+		self._contactsmodel = gtk.ListStore(
+			gobject.TYPE_STRING, # Contact Type
+			gobject.TYPE_STRING, # Contact Name
+			gobject.TYPE_STRING, # Contact ID
+		)
 		self._contactsviewselection = None
 		self._contactsview = widgetTree.get_widget("contactsview")
 
@@ -1466,14 +1337,11 @@ class ContactsView(object):
 		if displayContactSource:
 			textrenderer = gtk.CellRendererText()
 			self._contactColumn.pack_start(textrenderer, expand=False)
-			self._contactColumn.add_attribute(textrenderer, 'text', 0)
+			self._contactColumn.add_attribute(textrenderer, 'text', self.CONTACT_TYPE_IDX)
 		textrenderer = gtk.CellRendererText()
 		hildonize.set_cell_thumb_selectable(textrenderer)
 		self._contactColumn.pack_start(textrenderer, expand=True)
-		self._contactColumn.add_attribute(textrenderer, 'text', 1)
-		textrenderer = gtk.CellRendererText()
-		self._contactColumn.pack_start(textrenderer, expand=True)
-		self._contactColumn.add_attribute(textrenderer, 'text', 4)
+		self._contactColumn.add_attribute(textrenderer, 'text', self.CONTACT_NAME_IDX)
 		self._contactColumn.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
 		self._contactColumn.set_sort_column_id(1)
 		self._contactColumn.set_visible(True)
@@ -1602,8 +1470,9 @@ class ContactsView(object):
 					self._isPopulated = False
 					self._errorDisplay.push_exception_with_lock()
 				for contactId, contactName in contacts:
-					contactType = (addressBook.contact_source_short_name(contactId), )
-					self._contactsmodel.append(contactType + (contactName, "", contactId) + ("", ))
+					contactType = addressBook.contact_source_short_name(contactId)
+					row = contactType, contactName, contactId
+					self._contactsmodel.append(row)
 
 				with gtk_toolbox.gtk_lock():
 					self._contactsview.set_model(self._contactsmodel)
@@ -1647,8 +1516,8 @@ class ContactsView(object):
 			if not itr:
 				return
 
-			contactId = self._contactsmodel.get_value(itr, 3)
-			contactName = self._contactsmodel.get_value(itr, 1)
+			contactId = self._contactsmodel.get_value(itr, self.CONTACT_ID_IDX)
+			contactName = self._contactsmodel.get_value(itr, self.CONTACT_NAME_IDX)
 			try:
 				contactDetails = self._addressBook.get_contact_details(contactId)
 			except Exception, e:
